@@ -25,8 +25,6 @@ use Badger::Class
     as_text      => 'path',
     is_true      => 1,
     constant     => {
-        is_file      => 0,
-        is_directory => 0,
         type         => 'Path',
         STAT_PATH    => 17,         # offset in extended stat fields
     },
@@ -49,18 +47,18 @@ our $FILESYSTEM  = 'Badger::Filesystem';
 our $TIMESTAMP   = 'Badger::Timestamp';
 our $MATCH_EXT   = qr/\.([^\.]+)$/;       # TODO: is this filesystem-specific?
 our @VDN_FIELDS  = @Badger::Filesystem::Base::VDN_FIELDS;
-our @STAT_FIELDS = qw( device inode mode links user group device_type 
+our @STAT_FIELDS = qw( device inode mode links user group device_type
                        size atime mtime ctime block_size blocks
                        readable writeable executable owner );
-our $STAT_FIELD  = { 
+our $STAT_FIELD  = {
     # In here we'll store the map from stat field name to number
     #   device => 0,
-    #   inode  => 1, 
+    #   inode  => 1,
     #   ...etc...
 };
 
 our $TS_FIELD = {
-    # On the left we have the timestamp methods we want to generate as 
+    # On the left we have the timestamp methods we want to generate as
     # wrappers around the stat fields listed on the right.
     created  => 'ctime',
     accessed => 'atime',
@@ -71,11 +69,11 @@ our $TS_FIELD = {
 my $n = 0;
 
 class->methods(
-    map { 
+    map {
         my $m = $n++;                       # new lexical variable for closure
         $STAT_FIELD->{ $_ } = $m;           # fill in $STAT_FIELD entry
         $_ => sub { $_[0]->stats->[$m] }    # generate subroutine
-    } 
+    }
     @STAT_FIELDS
 );
 
@@ -83,15 +81,15 @@ class->methods(
 # Badger::Timestamp objects for the atime, ctime and mtime stat values
 
 class->methods(
-    map { 
+    map {
         my $method = $_;                    # new lexical variable for closure
         my $stat   = $TS_FIELD->{ $_ };
         my $statno = $STAT_FIELD->{ $stat };
-        $method => sub { 
-            return $_[0]->{ $method } 
+        $method => sub {
+            return $_[0]->{ $method }
                 ||= $TIMESTAMP->new( $_[0]->stats->[$statno] )
         }
-    } 
+    }
     keys %$TS_FIELD
 );
 
@@ -100,7 +98,7 @@ class->methods(
 *is_dir    = \&is_directory;
 *dir       = \&directory;
 *vol       = \&volume;     # goes up to 11
-*ext       = \&extension;  
+*ext       = \&extension;
 *base_name = \&basename;
 *up        = \&parent;
 *meta      = \&metadata;
@@ -111,7 +109,7 @@ class->methods(
 sub new {
     my $class = shift; $class = ref $class || $class;
     my $args;
-    
+
     if (@_ == 1) {
         $args = ref $_[0] eq HASH ? shift
               : ref $_[0] eq ARRAY || ! ref $_[0] ? { path => shift }
@@ -120,14 +118,14 @@ sub new {
     else {
         $args = { @_ };
     }
-    
+
     # allow short aliases for various configuration options, including
     # those of directory/file subclasses to make life easy for them.
     $args->{ filesystem } ||= $args->{ fs  } if $args->{ fs  };
     $args->{ directory  } ||= $args->{ dir } if $args->{ dir };
     $args->{ volume     } ||= $args->{ vol } if $args->{ vol };
     my $self = bless { }, $class;
-    
+
     # maintain a reference to the filesystem that created us, if available,
     # but don't bother if we didn't get one - we can use the default
     $self->{ filesystem } = $args->{ filesystem } if $args->{ filesystem };
@@ -153,6 +151,18 @@ sub is_relative {
     shift->is_absolute ? 0 : 1;
 }
 
+sub is_file {
+    my $self = shift;
+    my $defn = $self->filesystem->definitive_read($self->{ path }) || return;
+    return -f $defn;
+}
+
+sub is_directory {
+    my $self = shift;
+    my $defn = $self->filesystem->definitive_read($self->{ path }) || return;
+    return -d $defn;
+}
+
 sub absolute {
     my $self = shift;
     return $self->is_absolute
@@ -164,8 +174,8 @@ sub relative {
     my $self = shift;
     my $fs   = $self->filesystem;
     my $path = $fs->join_directory(@_);
-    # If the path isn't already absolute then we merge it onto our 
-    # directory or path if directory is undefined.  By calling the 
+    # If the path isn't already absolute then we merge it onto our
+    # directory or path if directory is undefined.  By calling the
     # base() method, we allow the file subclass to return its
     # parent directory so that things Just Work[tm]
 #    $self->debug("relative path: $path   is_absolute?\n");
@@ -177,13 +187,13 @@ sub relative {
 sub definitive {
     my $self = shift;
     # use the definitive path from the last stat or fetch anew
-    return 
+    return
         ($self->{ stats } && $self->{ stats }->[STAT_PATH])
       || $self->filesystem->definitive($self->{ path });
 }
 
 sub collapse {
-    my $self = shift->absolute;
+    my $self = shift;
     my $fs   = $self->filesystem;
     $self->{ directory } = $fs->collapse_directory( $self->{ directory } );
     $self->{ path      } = $fs->join_path( @$self{ @VDN_FIELDS } );
@@ -215,12 +225,12 @@ sub base {
 sub parent {
     my $self   = shift;
     my $skip   = shift || 0;
-    my $parent = $self->{ parent } 
-             ||= $self->filesystem->directory( 
+    my $parent = $self->{ parent }
+             ||= $self->filesystem->directory(
                  $self->{ directory } ||= $self->path_up
              );
 
-    return 
+    return
         # don't return parents above the root
         $self->{ path } eq $parent->{ path } ? $self
         # delegate to parent if there are generations to skip
@@ -238,14 +248,14 @@ sub path_up {
         if $DEBUG;
 
     if (@$path > 1) {
-        # multiple items in path can be relative or absolute - we're not 
+        # multiple items in path can be relative or absolute - we're not
         # fussed.  e.g. /foo/bar ==> /foo  or  foo/bar ==> foo
         pop(@$path);
     }
     elsif (@$path == 1) {
         # if there's a single item in a path then it's either a single
-        # relative path item (e.g. 'foo' ==> ['foo']), in which case we 
-        # return the current working directory, or it's an empty item 
+        # relative path item (e.g. 'foo' ==> ['foo']), in which case we
+        # return the current working directory, or it's an empty item
         # indicating the root directory (e.g. '/' => ['']) in which case we
         # do nothing, because you can't go up from the root directory.
         if (length $path->[0]) {
@@ -256,7 +266,7 @@ sub path_up {
     else {
         $self->error("Invalid path (no elements)\n");
     }
-    
+
     return $fs->join_directory($path);
 }
 
@@ -292,15 +302,15 @@ sub stat {
     # the definitive path can be tagged on the end
 #    $self->{ definitive } = $stats->[STAT_PATH]
 #        if defined $stats->[STAT_PATH];
-        
-    return wantarray 
+
+    return wantarray
         ? @$stats
         :  $stats;
 }
 
 sub stats {
     my $stats = $_[0]->{ stats } ||= $_[0]->stat;
-    return wantarray 
+    return wantarray
         ? @$stats
         :  $stats;
 }
@@ -315,7 +325,7 @@ sub restat {
 sub permissions {
     shift->mode & 0777;
 }
-    
+
 sub chmod {
     my $self = shift;
     $self->filesystem->chmod_path($self->{ path }, @_);
@@ -341,7 +351,7 @@ sub filesystem {
     my $self = shift;
     return $self->class->any_var('FILESYSTEM')->prototype
         unless ref $self;
-    $self->{ filesystem } 
+    $self->{ filesystem }
         ||= $self->class->any_var('FILESYSTEM')->prototype;
 }
 
@@ -390,20 +400,20 @@ Badger::Filesystem::Path - generic fileystem path object
 
     # using Badger::Filesytem constructor subroutine
     use Badger::Filesystem 'Path';
-    
+
     # use native OS-specific paths:
     $path = Path('/path/to/something');
-    
+
     # or generic OS-independant paths
     $path = Path('path', 'to', 'something');
 
     # manual object construction
     use Badger::Filesystem::Path;
-    
+
     # positional arguments
     $path = Badger::Filesystem::Path->new('/path/to/something');
     $path = Badger::Filesystem::Path->new(['path', 'to', 'something']);
-    
+
     # named parameters
     $path = Badger::Filesystem::Path->new(
         path => '/path/to/something'
@@ -411,7 +421,7 @@ Badger::Filesystem::Path - generic fileystem path object
     $path = Badger::Filesystem::Path->new(
         path => ['path', 'to', 'something']
     );
-    
+
     # path inspection methods
     $path->path;                    # current path
     $path->base;                    # parent directory or path itself
@@ -431,14 +441,14 @@ Badger::Filesystem::Path - generic fileystem path object
     $path->absolute;                # relative to filesystem root
     $path->definitive;              # physical file location
     $path->collapse;                # resolve '.' and '..' in $path
-    
+
     # path comparison methods
     $path->above($another_path);    # $path is ancestor of $another_path
     $path->below($another_path);    # $path is descendant of $another_path
 
 =head1 INTRODUCTION
 
-This is the documentation for the C<Badger::Filesystem::Path> module. 
+This is the documentation for the C<Badger::Filesystem::Path> module.
 It defines a base class object for the L<Badger::Filesystem::File> and
 L<Badger::Filesystem::Directory> objects which inherit (and in some cases
 redefine) the methods described below.
@@ -449,26 +459,26 @@ with L<Badger::Filesystem::File> or L<Badger::Filesystem::Directory> objects.
 =head1 DESCRIPTION
 
 The C<Badger::Filesystem::Path> module defines a base class object for
-representing paths in a real or virtual file system. 
+representing paths in a real or virtual file system.
 
 You can create a generic path object (e.g. to represent a path that doesn't
 relate to a specific file or directory in a file system), using the C<Path>
 constructor method in L<Badger::Filesystem>.
 
     use Badger::Filesystem 'Path';
-    
+
     my $path = Path('/path/to/something');
 
 However in most cases you'll want to create a file or directory subclass
 object. The easiest way to do that is like this:
 
     use Badger::Filesystem 'File Path';
-    
+
     my $file = File('/path/to/file');
     my $dir  = Dir('/path/to/dir');
 
 If you're concerned about portability to other operating systems and/or
-file systems, then you can specify paths as a list or reference to a list 
+file systems, then you can specify paths as a list or reference to a list
 of component names.
 
     my $file = File('path', 'to', 'file');
@@ -484,7 +494,7 @@ text string or reference to list of path components.
 
     # single text string
     $path = Badger::Filesystem::Path->new('/path/to/something');
-    
+
     # reference to list
     $path = Badger::Filesystem::Path->new(['path', 'to', 'something']);
 
@@ -507,16 +517,16 @@ path objects for you, using either the L<path()|Badger::Filesystem/path()>
 method, or the L<Path()|Badger::Filesystem/Path()> subroutine.
 
     use Badger::Filesystem 'FS Path';
-    
+
     # FS is alias for 'Badger::Filesystem'
     # Path() is constructor subrooutine
     my $path;
-    
+
     # using the path() method
     $path = FS->path('/path/to/something');
     $path = FS->path('path', 'to', 'something');
     $path = FS->path(['path', 'to', 'something']);
-    
+
     # using the Path() subroutine
     $path = Path('/path/to/something');
     $path = Path('path', 'to', 'something');
@@ -527,7 +537,7 @@ The examples that follow will use the C<Path()> constructor subroutine.
 =head2 init(\%config)
 
 Default initialisation method which subclasses (e.g.
-L<Badger::Filesystem::Directory> and L<Badger::Filesystem::File>) can 
+L<Badger::Filesystem::Directory> and L<Badger::Filesystem::File>) can
 redefine.
 
 =head2 path()
@@ -549,9 +559,9 @@ Returns an absolute representation of the path, relative to the C<$base>
 path passed as an argument, or the current working directory if C<$base>
 is not specified.
 
-    # assume cwd is /foo/bar, 
+    # assume cwd is /foo/bar,
     my $path = Path('/baz/bam');
-    
+
     print $path->absolute;                  # /foo/bar/baz/bam
     print $path->absolute('/wiz');          # /wiz/baz/bam
 
@@ -561,9 +571,9 @@ Returns a relative representation of the path, relative to the C<$base>
 path passed as an argument, or the current working directory if C<$base>
 is not specified.
 
-    # assume cwd is /foo/bar, 
+    # assume cwd is /foo/bar,
     my $path = Path('/foo/bar/baz/bam');
-    
+
     print $path->relative;                  # /baz/bam
     print $path->relative('/foo');          # /bar/baz/bam
 
@@ -597,7 +607,7 @@ trailing slashes (or whatever the appropriate directory separator is for your
 filesystem) from the end of an absolute path.
 
 In the case of directories, implemented by the
-L<Badger::Filesystem::Directory> subclass, a trailing slash (or relevant 
+L<Badger::Filesystem::Directory> subclass, a trailing slash (or relevant
 separator for your filesystem) will be added.
 
     print Dir('/foo/bar')->canonical;          # /foo/bar/
@@ -611,12 +621,12 @@ L<Badger::Filesystem>.
 Reduces the path to its simplest form by resolving and removing any C<.>
 (current directory) and C<..> (parent directory) components (or whatever the
 corresponding tokens are for the current and parent directories of your
-filesystem). 
+filesystem).
 
     my $path = Path('/foo/bar/../baz')->collapse;
     print $path;   # /foo/baz
 
-See the L<collapse_dir()|Badger::Filesystem/collapse_dir()> method in 
+See the L<collapse_dir()|Badger::Filesystem/collapse_dir()> method in
 L<Badger::Filesystem> for further information.
 
 =head2 above($child)
@@ -670,8 +680,8 @@ directory for a path.
 
 A numerical argument can be provided to indicate the number of generation
 you want to skip.  A value of C<0> is the same as providing no argument - it
-returns the parent.  A value of C<1> skips the parent and returns the 
-grand-parent, and so on. 
+returns the parent.  A value of C<1> skips the parent and returns the
+grand-parent, and so on.
 
     Path->('/foo/bar/baz/bam')->parent(2);  # path object for /foo
 
@@ -684,7 +694,7 @@ The root directory will be returned if you try to skip too many generations.
 This returns a text string representing the parent of a path.  If the path
 contains multiple items (e.g. '/foo/bar' or 'foo/bar') then the last item
 will be removed (e.g. resulting in '/foo' or 'foo' respectively).  If an
-absolute path contains one item or none (e.g. '/foo' or '/') then the 
+absolute path contains one item or none (e.g. '/foo' or '/') then the
 root directory ('/') will be returned.  A relative path with only one item
 (e.g. 'foo') is assumed to be relative to the current working directory
 which will be returned (e.g. '/path/to/current/dir').
@@ -715,7 +725,7 @@ subclasses and not base class paths.
 
     $dir->must_exist(1);                    # create if it doesn't
 
-=head2 create() 
+=head2 create()
 
 In the base class this will method will throw an error. You can't physically
 create an abstract path unless you know what kind of concrete entity (e.g.
@@ -736,7 +746,7 @@ This method changes the file permissions on a file or directory.
 =head2 stat()
 
 Performs a filesystem C<stat> on the path and returns a list (in list
-context), or a reference to a list (in scalar context) containing the 13 
+context), or a reference to a list (in scalar context) containing the 13
 information elements.
 
     @list = $path->stat;                    # list context
@@ -772,21 +782,21 @@ this method returns four additional flags.
 
 =head2 stats()
 
-A wrapper around the L<stat()> method which caches the results to avoid 
+A wrapper around the L<stat()> method which caches the results to avoid
 making repeated filesystem calls.
 
     @list = $path->stats;                   # list context
     $list = $path->stats;                   # scalar context
 
-Note that the L<accessed()>, L<created()> and L<modified()> methods also 
-cache the L<Badger::Timestamp> objects they create to represent the 
+Note that the L<accessed()>, L<created()> and L<modified()> methods also
+cache the L<Badger::Timestamp> objects they create to represent the
 access, creation and modification times respectively.
 
 =head2 restat()
 
-Clears any cached values stored by the L<stats()>, L<accessed()>, 
+Clears any cached values stored by the L<stats()>, L<accessed()>,
 L<created()> and L<modified()> methods and calls L<stats()> to reload
-(and re-cache) the data from a L<stat()> call. 
+(and re-cache) the data from a L<stat()> call.
 
 =head2 device()
 
@@ -798,12 +808,12 @@ Returns the inode number for the file.  See L<stat()>.
 
 =head2 mode()
 
-Returns the file mode for the file.  Note that this contains both the 
+Returns the file mode for the file.  Note that this contains both the
 file type and permissions.  See L<stat()>.
 
 =head2 permissions() / perms()
 
-Returns the file permissions.  This is equivalent to 
+Returns the file permissions.  This is equivalent to
 C<< $file->mode & 0777 >>.
 
 =head2 links()
@@ -834,7 +844,7 @@ See L<stat()>.
 =head2 accessed()
 
 Returns a L<Badger::Timestamp> object for the L<atime()> value.  This object
-will auto-stringify to produce an ISO-8601 formatted date.  You can also 
+will auto-stringify to produce an ISO-8601 formatted date.  You can also
 call various methods to access different parts of the time and/or date.
 
     print $file->accessed;              # 2009/04/20 16:25:00
@@ -891,7 +901,7 @@ owner of the current process), false if not.  See L<stat()>.
 Returns a true value if the file is executable by the current user (i.e. the
 owner of the current process), false if not.  See L<stat()>.
 
-=head2 owner()                
+=head2 owner()
 
 Returns a true value if the file is owned by the current user (i.e. the
 owner of the current process), false if not.  See L<stat()>.
@@ -900,42 +910,42 @@ owner of the current process), false if not.  See L<stat()>.
 
 Returns a reference to a L<Badger::Filesystem> object, or the name of the
 filesystem class (e.g. L<Badger::Filesystem> or a subclass) that created
-the path object.  If this is undefined then the default value defined in 
+the path object.  If this is undefined then the default value defined in
 the L<$FILESYSTEM> class variable is returned.   Unless you've changed it,
 or re-defined it in a subclass, this value will be C<Badger::Filesystem>.
 
-The end result is that you can use the C<filesystem> method to access a 
+The end result is that you can use the C<filesystem> method to access a
 L<Badger::Filesystem> object or class through which you can perform other
-filesystem related operations.  This is used internally by a number of 
+filesystem related operations.  This is used internally by a number of
 method.
 
     # access filesystem via existing path
     $path->filesystem->dir('/a/new/directory/object');
-    
+
     # same as
     Badger::Filesystem->dir('/a/new/directory/object');
 
 =head2 visit($visitor)
 
-Entry point for a filesystem visitor to visit a filesystem path.  A 
+Entry point for a filesystem visitor to visit a filesystem path.  A
 reference to a L<Badger::Filesystem::Visitor> object (or subclass) should
-be passed as the first argument.  
+be passed as the first argument.
 
     use Badger::Filesystem::Visitor;
-    
+
     my $visitor = Badger::Filesystem::Visitor->new( recurse => 1 );
     $path->visit($visitor);
 
 Alternately, a list or reference to a hash array of named parameters may be
 provided. These will be used to instantiate a new
 L<Badger::Filesystem::Visitor> object (via the L<Badger::Filesystem>
-L<visitor()|Badger::Filesystem/visitor()> method) which will then be 
+L<visitor()|Badger::Filesystem/visitor()> method) which will then be
 applied to the path.  If no arguments are passed then a visitor is created
 with a default configuration.
 
     # either list of named params
     $path->visit( recurse => 1 );
-    
+
     # or reference to hash array
     $path->visit({ recurse => 1});
 
@@ -955,14 +965,14 @@ method to call other visitor methods.
 
 This is a special case of the L<accept()> method which subclasses (e.g.
 L<directory|Badger::Filesystem::Directory>) use to differentiate between the
-initial entry point of a visitor and subsequent visits to directories 
+initial entry point of a visitor and subsequent visits to directories
 contained therein.  In the base class it simply delegates to the L<accept()>
 method.
 
 =head2 collect(\%params)
 
-This is a short-cut to call the L<visit()> method and then the 
-L<collect()|Badger::Filesystem::Visitor/collect()> method on the 
+This is a short-cut to call the L<visit()> method and then the
+L<collect()|Badger::Filesystem::Visitor/collect()> method on the
 L<Badger::Filesystem::Visitor> object returned.
 
     # short form
@@ -1034,17 +1044,17 @@ full path.
 
 =head2 basename() / base_name()
 
-Returns the filename I<without> the file extension following the final 
-C<.> in the path.  This works (for some definition of "works") in the 
+Returns the filename I<without> the file extension following the final
+C<.> in the path.  This works (for some definition of "works") in the
 C<Badger::Filesystem::Path> base class by looking at the path L<name()>,
-if defined, or the full C<path> if not.  Note that this will produce 
-unexpected results in some cases due to the fact that the base class 
+if defined, or the full C<path> if not.  Note that this will produce
+unexpected results in some cases due to the fact that the base class
 does not define a value for L<name()>.  e.g.
 
     print Path('/foo/bar.txt')->basename;       # /foo/bar
 
 However, in most cases you would be using this through a
-L<Badger::Filesystem::File> subclass which will product the correct 
+L<Badger::Filesystem::File> subclass which will product the correct
 results.
 
     print File('/foo/bar.txt')->basename;       # bar
@@ -1068,7 +1078,7 @@ in L<Badger::Filesystem> for further information.
 
 =head1 SEE ALSO
 
-L<Badger::Filesystem>, 
+L<Badger::Filesystem>,
 L<Badger::Filesystem::File>,
 L<Badger::Filesystem::Directory>,
 L<Badger::Filesystem::Visitor>.
@@ -1083,4 +1093,3 @@ L<Badger::Filesystem::Visitor>.
 #
 # vim: expandtab shiftwidth=4:
 # TextMate: should support split pane editing
-
