@@ -14,6 +14,7 @@
 package Badger::Hub;
 
 use Badger::Debug ':dump';
+use Badger::Config;
 use Badger::Class
     version   => 0.01,
     debug     => 0,
@@ -21,57 +22,146 @@ use Badger::Class
     import    => 'class',
     auto_can  => 'auto_can',
     constants => 'HASH ARRAY REFS PKG',
-    utils     => 'blessed',
+    utils     => 'blessed params is_object',
     words     => 'COMPONENTS DELEGATES COMP_CACHE DELG_CACHE',
+    constant  => {
+        CONFIG_MODULE => 'Badger::Config',
+    },
     messages => {
         no_module  => 'No %s module defined.',
         bad_method => "Invalid method '%s' called on %s at %s line %s",
     };
 
-use Badger::Config;
-our $CONFIG     = 'Badger::Config';
-our $COMPONENTS = { };
-our $DELEGATES  = { };
-our $LOADED     = { };
+our $COMPONENTS    = { };
+our $RESOURCES     = { };
+our $DELEGATES     = { };
+our $LOADED        = { };
+#our $COLLECTIONS = [qw( components resources delegates )];
 
 
 sub init {
+    my ($self, $config) = @_;
+    $self->init_hub($config);
+}
+
+sub init_hub {
+    my ($self, $config) = @_;
+    $self->init_config($config);
+    $self->init_items($config);
+    # $self->init_collections($config);     # init on demand
+}
+
+sub init_config {
     my ($self, $args) = @_;
+    my $class = $self->class;
 
     # We're looking for a specific 'config' item which the user can provide to
     # points to a master configuration object or class name.  We default to the 
     # value in the $CONFIG package variable, which in this case is Badger::Config,
     # but could be re-defined by a subclass to be something else.
-    my $config = delete($args->{ config }) || $self->class->any_var('CONFIG');
-    class($config)->load unless ref $config;
+
+    my $config = delete($args->{ config }) 
+        || $self->class->any_var('CONFIG')
+        || { };
+
+    my $module = delete($args->{ config_module })
+        || $class->any_var('CONFIG_MODULE')
+        || $self->CONFIG_MODULE;
+
+    if (is_object($module, $config)) {
+        # $config is already a config module object of the right heritage
+        $self->debug("config is already a $module object") if DEBUG;
+    }
+    else {
+        if (ref $config eq HASH) {
+            # cool
+        }
+        elsif (! ref $config) {
+            # $config is a module name
+            $module = $config;
+            $config = { };
+        }
+        else {
+            return $self->error_msg( invalid => config => $config );
+        }
+        $self->debug("loading and instantiating a custom $module config object") if DEBUG;
+        class($module)->load;
+        $config = $module->new($config);
+    }
+
+    $self->{ config } = $config;
+    $self->debug("hub config: $self->{ config }\n") if DEBUG;
+}
+
+sub init_items {
+    my ($self, $args) = @_;
 
     # merge all values in $CONFIG_ITEMS in with $args->{ items };
     $args->{ items } = $self->class->list_vars( 
         CONFIG_ITEMS => delete($args->{ config_items }), $args->{ items }
     );
 
-    $self->debug("hub config items: ", $self->dump_data($args->{ items })) if DEBUG;
-    
-    $config = $config->new($args) unless blessed $config;
-    $self->{ config } = $config;
+    $self->debug(
+        "hub config items: ", 
+        $self->dump_data($args->{ items })
+    ) if DEBUG;
 
-    $self->debug("hub config: $self->{ config }\n") if DEBUG;
     return $self;
 }
 
-
-sub config {
-    my $self = shift;
-    $self = $self->prototype(@_) unless ref $self;
-    return $self->{ config };
+sub init_collections {
+    my ($self, $config) = @_;
+    $self->init_components($config);
+    $self->init_resource($config);
+    $self->init_delegates($config);
 }
 
+sub init_components {
+    shift->init_collection( components => @_ );
+}
+
+sub init_resources {
+    shift->init_collection( resources => @_ );
+}
+
+sub init_delegates {
+    shift->init_collection( delegates => @_ );
+}
+
+sub init_collection {
+    my ($self, $type) = @_;
+    my $TYPE = uc $type;
+    my $cfg  = $self->config($type);
+    my $bits = $self->{ $type } = $self->class->hash_vars( 
+        $TYPE => $self->config($type)
+    );
+    $self->debug("init_collection: $type: ", $self->dump_data($bits)) if DEBUG;
+    return $bits;
+}
+
+sub configure {
+    my ($self, $config) = self_params(@_);
+    my $item;
+
+    if ($item = delete $config->{ components }) {
+        $self->components($item);
+    }
+    if ($item = delete $config->{ resources }) {
+        $self->resources($item);
+    }
+    if ($item = delete $config->{ delegates }) {
+        $self->delegates($item);
+    }
+}
+
+#-----------------------------------------------------------------------------
+# comment
+#-----------------------------------------------------------------------------
 
 sub components {
-    my $self  = shift;
-    my $class = $self->class;
-    my $comps = $class->var(COMP_CACHE) 
-             || $class->var(COMP_CACHE => $class->hash_vars(COMPONENTS));
+    my $self  = shift->prototype;
+    my $comps = $self->{ components }
+             || $self->init_components;
 
     if (@_) {
         my $args = ref $_[0] eq HASH ? shift : { @_ };
@@ -85,6 +175,7 @@ sub components {
 sub component {
     my $self  = shift;
     my $comps = $self->components;
+    $self->debug("components: ", $self->dump_data($comps)) if DEBUG;
     return @_
         ? $comps->{ $_[0] }
         : $comps;
@@ -92,10 +183,10 @@ sub component {
 
 
 sub delegates {
-    my $self  = shift;
+    my $self  = shift->prototype;
     my $class = $self->class;
-    my $delgs = $class->var(DELG_CACHE) 
-             || $class->var(DELG_CACHE => $class->hash_vars(DELEGATES));
+    my $delgs = $self->{ delegates }
+             || $self->init_delegates;
 
     if (@_) {
         my $args = ref $_[0] eq HASH ? shift : { @_ };
@@ -109,6 +200,7 @@ sub delegates {
 sub delegate {
     my $self   = shift;
     my $delegs = $self->delegates;
+    $self->debug("delegates: ", $self->dump_data($delegs)) if DEBUG;
     return @_
         ? $delegs->{ $_[0] }
         : $delegs;
@@ -120,15 +212,15 @@ sub auto_can {
     my $target;
 
     if ($target = $self->component($name)) {
-        $self->debug("creating component method for $name")  if DEBUG;
+        $self->debug("creating component method in $self for $name")  if DEBUG;
         return $self->auto_component( $name => $target );
     }
     elsif ($target = $self->delegate($name)) {
-        $self->debug("creating delegate method for $name") if DEBUG;
+        $self->debug("creating delegate method in $self for $name") if DEBUG;
         return $self->auto_delegate( $name => $target );
     }
     elsif (DEBUG) {
-        $self->debug("no component or delegate found for $name");
+        $self->debug("no component or delegate found in $self for $name");
     }
         
     return undef;
@@ -189,57 +281,97 @@ sub auto_delegate {
 # $class->name()).  This is merged with $params.
 
 sub construct {
-    my $self = shift;
-    my $name = shift;
-    my $args = @_ && ref($_[0]) eq HASH ? shift : { @_ };
-
-    $self->debug("construct($name)") if DEBUG;
-    
-    # $NAME pkg var can be a module name or hash ref with 'module' item
-    my $pkgvar = $self->class->any_var(uc $name);
-    my $pkgmod = ref $pkgvar eq HASH ? $pkgvar->{ module } : $pkgvar;
-    my $config = $self->{ config };
-    my $params;
-    my $method;
-
-    if ($config && ref $config eq HASH) {
-        # $self->{ config } can be a hash ref with a $name item
-        $params = $config->{ $name };
-    }
-    elsif (blessed $config && ($method = $self->{ config }->can($name))) {
-        # $self->{ config } can be an object with a $name method which we call
-        $params = $method->($config);
-    }
-    else {
-        # no local config data so we'll fall back on a package variable
-        $params = $pkgvar;
-    }
-    
-    # if $params isn't defined then we default to the entire $config hash
-    $params ||= $config;
-
-    # if $config isn't a hash then it's the name of the module to use
-    $params = { module => $params } unless ref $params eq HASH;
+    my $self   = shift;
+    my $name   = shift;
+    my $config = $self->config($name, @_);
 
     $self->debug("$name module config: ", $self->dump_data($config)) if DEBUG;
     
     # see if a module name is specified in $args, config hash or use $pkgmod
-    my $module = $args->{ module } ||= $params->{ module } ||= $pkgmod
+    my $module = $config->{ module }
         || return $self->error_msg( no_module => $name );
 
-    $self->debug("$name module: $module") if DEBUG;
-
     # load the module
     class($module)->load;
 
-    # add any extra arguments to the config hash
-    $params = { %$params, %$args } if %$args;
-
-    $self->debug("$name merged config: ", $self->dump_data($params)) if DEBUG;
-
-    return $module->new($params);
+    return $module->new($config);
 }
 
+sub config {
+    my $self   = shift->prototype;
+    my $config = $self->{ config };     return $config unless @_;
+    my $name   = shift;
+    my $params = params(@_);
+    my $defaults;
+    my $method;
+
+
+    if ($config && ref $config eq HASH) {
+        # $self->{ config } can be a hash ref with a $name item
+        $defaults = $config->{ $name };
+    }
+    elsif (blessed $config && ($method = $config->can('get'))) {
+        # $self->{ config } can be an object with a get($name) method which we call
+        $self->debug("calling config->get($name)") if DEBUG;
+        $defaults = $method->($config, $name);
+    }
+    elsif (blessed $config && ($method = $config->can($name))) {
+        # $self->{ config } can be an object with a $name method which we call
+        $self->debug("calling config->$name") if DEBUG;
+        $defaults = $method->($config);
+    }
+    # no defaults
+    return $self->no_config($name, $params)
+        unless $defaults;
+
+    $self->debug(
+        "config for $name: DEFAULTS: ", 
+        $self->dump_data($defaults),
+        "\nPARAMS: ", $self->dump_data($params)
+    ) if DEBUG;
+
+    return {
+        %$defaults,
+        %$params
+    };
+}
+
+sub no_config {
+    my ($self, $name, $params) = @_;
+
+    $self->debug(
+        "no_config for $name: ", 
+        "PARAMS: ", $self->dump_data($params)
+    ) if DEBUG;
+
+    # TODO: option to make this a failure
+    return  $self->pkgvar_config($name, $params)
+        ||  ($params ? { %$params } : { });
+}
+
+sub pkgvar_config {
+    my ($self, $name, $params) = @_;
+    my $pkgvar = $self->class->any_var(uc $name) || return $params;
+    my $config;
+
+    if (ref $pkgvar eq HASH) {
+        # package variable can be a hash ref of config options
+        $config = $pkgvar;
+    }
+    else {
+        # or the name of a module
+        $config = { 
+            module => $pkgvar 
+        };
+    };
+
+    return {
+        %$config,
+        %$params
+    };
+}
+
+
 
 #------------------------------------------------------------------------
 # destroy()
